/*
	Bootloader for the TaltOS
	on the Philips Chip LPC2106

	By Peter Eordegh
*/


//Component table pointer
#define TALTOS_COMPONENT_TABLE_PTR (0x0001C000)	//Sector 14 (sector 15 is the remapped bootloader)
#define TALTOS_COMPONENT_TABLE_SIZE (0x2000)	//This is a full sector size

//LPC2106 sepcific flash memory parameters
#define LPC2106_FLASH_MEMORY_SIZE (0x20000)		//All the flash memory size
#define LPC2106_SECTOR_SIZE (0x2000)			//Sector size 8192 bytes
#define LPC2106_MINDATA (0x200)					//1/16 part of the sector can be written 512 bytes
#define LPC2106_RAM_TOP_PTR (0x4000FFE0)

//Flash operation buffer
#define TALTOS_FLASH_OP_BUFFER_SIZE (0x2000)	//This is a full sector size
#define TALTOS_FLASH_OP_BUFFER_PTR (LPC2106_RAM_TOP_PTR-TALTOS_FLASH_OP_BUFFER_SIZE)

//Input data buffer
#define TALTOS_INPUT_BUFFER_SIZE (0x1000)		//This is a half of a sector size
#define TALTOS_INPUT_BUFFER_PTR (TALTOS_FLASH_OP_BUFFER_PTR-TALTOS_INPUT_BUFFER_SIZE)

//TaltOS IDs
#define TALTOS_ID1 (0x0000014E)
#define TALTOS_ID2 (0x0000014E)

//Command Strings (unsigned longs)
#define TALTOS_COMP_BEGIN	(0x2A62632A)	// *cb* : Component Begin
#define TALTOS_COMP_END		(0x2A65632A)	// *ce* : Component End
#define TALTOS_COMP_LIST	(0x2A6C632A)	// *cl* : Component List
#define TALTOS_REM_COMP		(0x2A63722A)	// *rc* : Remove Component
#define TALTOS_RUN_INIT		(0x2A69722A)	// *ri* : Run (component) Init

#define TALTOS_ERASED (0x00000000)
#define TALTOS_EMPTY (0xFFFFFFFF)
#define TALTOS_COMPID_END (0x4E)

//Debug Macro
#define DEBUG(X) REG8(UART0_THR) = X;

typedef struct
{
	unsigned long ID1;
	unsigned long ID2;
	unsigned long* address;
	unsigned long size;
}	TaltOS_reg_entity;
